/*
File name  : FHN_LCD4_LIB.C
Author     : fhn
Date       : 07-Feb-2019
MCU        : Any AVR 
*/

#include <avr/io.h>
#include <util/delay.h>

//=============================================================================
#define LCD_PORT				PORTC
#define LCD_REG					DDRC
#define LCD_DATA_BITS			0b11110000// 4 MSB
#define LCD_ENABLE				0b00001000
#define LCD_RS					0b00000100
uint8_t LCD_COL=1;
uint8_t LCD_ROW=1;

void LCD_en ( )
{
    LCD_PORT |= LCD_ENABLE; 
    _delay_us(50);	
   LCD_PORT &= (~LCD_ENABLE ); 
   _delay_us(30);	
}
//=============================================================================
void LCD_wrNib ( unsigned char nibble , unsigned char RS)
{

	unsigned char nibl = nibble;
	if (nibl<16)
		nibl = (nibl<<4);

	LCD_PORT &= 0x0F; 
	LCD_PORT |= (nibl&0xF0); 

	if (RS)
		LCD_PORT |= LCD_RS;
	else
		LCD_PORT &= (~LCD_RS);

	LCD_en ( );
}

//=============================================================================
void LCD_wrCom ( unsigned char cmd )
{
	LCD_wrNib ( cmd & 0xF0 , 0);
	LCD_wrNib ( cmd & 0x0F , 0);
}
//=============================================================================
void LCD_clr ( )
{
	LCD_COL=1;
	LCD_ROW=1;
	LCD_wrCom ( 0x01 );
	_delay_ms(3);
}
//=================================================
void LCD_RC ( unsigned char Row, unsigned char Col )
{
	if (Col<1 || Col>16)
		return;
	LCD_ROW = Row;
	if ( Row<1 || Row>2 )
		LCD_ROW = 1;
	LCD_COL = Col;
	
	switch (LCD_ROW)
	{
	case 2:		LCD_wrCom ( 0xC0 + Col-1 );	break;
	case 1:
	default:	LCD_wrCom ( 0x80 + Col-1 );	break;
	}
	_delay_ms ( 3 );
}
//=============================================================================
void LCD_start( ) 
{
	unsigned char initval = 0x20; 

	_delay_ms ( 50 );
	LCD_wrNib  ( 0x30 , 0);
	_delay_ms ( 20 ); 
	LCD_wrNib ( 0x30 , 0);
	_delay_us ( 200 );
	LCD_wrNib  ( 0x30 , 0);
	_delay_ms ( 100 );
	
	LCD_wrNib  ( 0x20 , 0 );
	_delay_ms ( 25 );

	initval |= 0b00001000;  
	initval |= 0b00000100;  
	LCD_wrCom ( initval ); 
	_delay_ms ( 25 );

	LCD_wrCom ( 0x0C ); 
	_delay_ms ( 25 );
	LCD_wrCom ( 0x06 ); 
	_delay_ms ( 250 );

}
//=============================================================================
void LCD_wrChr ( unsigned char ch )
{
	unsigned char c=ch;

	if ( LCD_COL>16 )
		return;

	LCD_wrNib ( c & 0xF0 , 1);
	LCD_wrNib ( c & 0x0F , 1);
	LCD_COL++;
}
//=============================================================================
void LCD_wrStr ( char s[] )
{
	for (int i=0; s[i]!=0; i++)
		{
		LCD_wrChr ( s[i] );
		_delay_us ( 20 ); 
		}
}
//=============================================================================

