/*
File name : FHN_RTC_DS1307_LIB.C
Author      : fhn
Date         : 07-Feb-2019
MCU        : Any AVR with TWI/I2C interface
*/

#include <avr/io.h>
#include <avr/pgmspace.h>

#include "FHN_TWI_LIB.C"


#define		SECONDS			RTC_REGISTER[0]
#define		MINUTES			RTC_REGISTER[1]
#define		HOURS			RTC_REGISTER[2]
#define		DAY    			RTC_REGISTER[3]
#define		DATE   			RTC_REGISTER[4]
#define		MONTH  			RTC_REGISTER[5]
#define		YEAR   			RTC_REGISTER[6]
#define		SQWAV  			RTC_REGISTER[7]

#define  DS1307_W_ADDR			0xD0
#define  DS1307_R_ADDR			0xD1


unsigned char RTC_REGISTER[8];


char RTC_TIM_STR[12]="HH:MM:SS"; 	
char RTC_DAT_STR[12]="DD-MM-YYYY";	
char *WEEKDAYSTR[] = {"SUN", "MON", "TUES", "WEDNES", "THURS", "FRI", "SATUR" };

unsigned char RTC_setStartAddress(void)
{
   unsigned char errcode;
   
   errcode = TWI_Start();
   if(errcode == 1)
   {
   	 TWI_Stop();
	 return 1;
   } 
   
   errcode = TWI_SendAddr(DS1307_W_ADDR);
   
   if(errcode == 1)
   {
	 TWI_Stop();
	 return 1;
   } 
   
   errcode = TWI_SendData(0x00);
   if(errcode == 1)
   {
	 TWI_Stop();
	 return 1;
   } 

   TWI_Stop();
   return 0;
}
//==================================================
unsigned char RTC_read(void)
{

  unsigned char errcode, i, data;
  
  errcode = TWI_Start();
   if(errcode == 1)
   {
   	 TWI_Stop();
	 return 1;
   } 
   
   errcode = TWI_SendAddr(DS1307_W_ADDR);
   
   if(errcode == 1)
   {
     TWI_Stop();
	 return 1;
   } 
   
   errcode = TWI_SendData(0x00);
   if(errcode == 1)
   {
	 TWI_Stop();
	 return 1;
   } 

    errcode = TWI_RepeatStart();
   if(errcode == 1)
   {
   	 TWI_Stop();
	 return 1;
   } 
   
    errcode = TWI_SendAddr(DS1307_R_ADDR);
   
   if(errcode == 1)
   {
	 TWI_Stop();
	 return 1;
   } 
 
    for(i=0;i<7;i++)
   {
      if(i == 6)  	
	   	  data = TWI_ReceiveDataWithNoAck();
	  else
	  	  data = TWI_ReceiveDataWithAck();
		  
   	  if(data == TWI_ERROR_CODE)
   	  {
			TWI_Stop();
	   		return 1;
   	  }
	  
	  RTC_REGISTER[i] = data;
	}
	
	TWI_Stop();
	return 0;
}	  
//==================================================
uint8_t RTC_getTime(void)
{
   uint8_t error;
   error = RTC_read();
   if(error) return 1;


   RTC_TIM_STR[8] = 0x00;	 
   RTC_TIM_STR[7] = (SECONDS & 0x0f) | 0x30;	
   RTC_TIM_STR[6] = ((SECONDS & 0x70) >> 4) | 0x30; 

   
   RTC_TIM_STR[4] = (MINUTES & 0x0f) | 0x30;
   RTC_TIM_STR[3] = ((MINUTES & 0x70) >> 4) | 0x30;

   
   RTC_TIM_STR[1] = (HOURS & 0x0f) | 0x30;	
   RTC_TIM_STR[0] = ((HOURS & 0x30) >> 4) | 0x30;

   return 0;
}
//==================================================
unsigned char RTC_getDate(void)
{
  unsigned char error;
  error = RTC_read();
  if(error) return 1;

  RTC_DAT_STR[11] = 0x00;
  RTC_DAT_STR[10] = 0x00;
  RTC_DAT_STR[9] = (YEAR & 0x0f) | 0x30;
  RTC_DAT_STR[8] = ((YEAR & 0xf0) >> 4) | 0x30;
  RTC_DAT_STR[7] = '0';
  RTC_DAT_STR[6] = '2';
//  RTC_DAT_STR[5] = '/';
  RTC_DAT_STR[4] = (MONTH & 0x0f) | 0x30;
  RTC_DAT_STR[3] = ((MONTH & 0x10) >> 4) | 0x30;
//  RTC_DAT_STR[2] = '/';
  RTC_DAT_STR[1] = (DATE & 0x0f) | 0x30;
  RTC_DAT_STR[0] = ((DATE & 0x30) >> 4) | 0x30;
  return 0;
}  
//==================================================
void RTC_SetDateTime ( char yy, char mm, char dd, char hh, char mn, char ss )
{
  RTC_DAT_STR[11] = 0x00;		RTC_DAT_STR[10] = 0x00;
  RTC_DAT_STR[9] = yy%16;		RTC_DAT_STR[8] = yy/16;
  RTC_DAT_STR[7] = '0';			RTC_DAT_STR[6] = '2';
  RTC_DAT_STR[5] = '/';
  RTC_DAT_STR[4] = mm%16;		RTC_DAT_STR[3] = mm/16;
  RTC_DAT_STR[2] = '/';
  RTC_DAT_STR[1] = dd%16;		RTC_DAT_STR[0] = dd/16;

   RTC_TIM_STR[8] = 0x00;	  
   RTC_TIM_STR[7] = ss%16;	 
   RTC_TIM_STR[6] = ss/16; 
   RTC_TIM_STR[5] = ':';
   
   RTC_TIM_STR[4] = mn%16;
   RTC_TIM_STR[3] = mn/10;
   RTC_TIM_STR[2] = ':'; 
   
   RTC_TIM_STR[1] = hh%16;	
   RTC_TIM_STR[0] = hh/16;
}
//=====================================================
void RTC_UpdateRegisters(void)
{
  SECONDS = ((RTC_TIM_STR[6] & 0x07) << 4) | (RTC_TIM_STR[7] & 0x0f);
  MINUTES = ((RTC_TIM_STR[3] & 0x07) << 4) | (RTC_TIM_STR[4] & 0x0f);
  HOURS = ((RTC_TIM_STR[0] & 0x03) << 4) | (RTC_TIM_STR[1] & 0x0f);  
  DAY = RTC_DAT_STR[10];
  DATE = ((RTC_DAT_STR[0] & 0x03) << 4) | (RTC_DAT_STR[1] & 0x0f);
  MONTH = ((RTC_DAT_STR[3] & 0x01) << 4) | (RTC_DAT_STR[4] & 0x0f);
  YEAR = ((RTC_DAT_STR[8] & 0x0f) << 4) | (RTC_DAT_STR[9] & 0x0f);
}  
//==================================================
unsigned char RTC_WriteTime( char tmstr[] )
{
  unsigned char errcode, i;
  for ( int p=0; p<8 && tmstr[p]>0; p++)
  	RTC_TIM_STR[p] = tmstr[p];
  	RTC_UpdateRegisters();
   errcode = TWI_Start();
   if(errcode == 1)
   {
   	 TWI_Stop();
	 return(1);
   } 
   
   errcode = TWI_SendAddr(DS1307_W_ADDR);
   
   if(errcode == 1)
   {
	 TWI_Stop();
	 return(1);
   } 
   
   errcode = TWI_SendData(0x00);
   if(errcode == 1)
   {
	 TWI_Stop();
	 return(1);
   } 

    for(i=0;i<3;i++)
    {
	  errcode = TWI_SendData(RTC_REGISTER[i]);  
   	  if(errcode == 1)
   	  {
			TWI_Stop();
	   		return(1);
   	  }
    }
	
	TWI_Stop();
	return(0);
}
//==================================================
unsigned char RTC_WriteDate( char dtstr[] )
{
  unsigned char errcode, i;
  for ( int p=0; p<10 && dtstr[p]>0; p++)
  	RTC_DAT_STR[p] = dtstr[p];
  RTC_UpdateRegisters();
   errcode = TWI_Start();
   if(errcode == 1)
   {
   	 TWI_Stop();
	 return(1);
   } 
   
   errcode = TWI_SendAddr(DS1307_W_ADDR);
   
   if(errcode == 1)
   {
	 TWI_Stop();
	 return(1);
   } 
   
   errcode = TWI_SendData(0x03);
   if(errcode == 1)
   {
	 TWI_Stop();
	 return(1);
   } 
	RTC_REGISTER[7]=0b10010000; 
    for(i=3;i<=7;i++)
    {
	  errcode = TWI_SendData(RTC_REGISTER[i]);  
   	  if(errcode == 1)
   	  {
		TWI_Stop();
   		return(1);
   	  }
    }
	
	TWI_Stop();
	return(0);
}
//==================================================
