/*
File name : FHN_TWI_LIB.C
Author      : fhn
Date        : 07-02-2019
MCU       : any AVR having SDA and SDC pins with 4.7 to 10K pull-up resistances
*/

#include <avr/io.h>
#ifndef	FHN_TWI_LIB
#define	FHN_TWI_LIB

#define	 TWI_START					0x08
#define  TWI_REPEAT_START			0x10
#define  TWI_ARB_LOST				0x38
#define  TWI_MASTER_TX_SLAVE_ACK	0x18
#define  TWI_MASTER_TX_SLAVE_NACK	0x20
#define  TWI_MASTER_TX_DATA_ACK		0x28
#define  TWI_MASTER_TX_DATA_NACK	0x30
#define  TWI_MASTER_RX_SLAVE_ACK	0x40
#define  TWI_MASTER_RX_SLAVE_NACK	0x48
#define  TWI_MASTER_RX_DATA_ACK		0x50
#define  TWI_MASTER_RX_DATA_NACK	0x58

#define  TWI_ERROR_CODE				0x7E
//===============================================	
unsigned char TWI_Start(void)
{
 
	TWCR = (1<<TWINT)|(1<<TWSTA)|(1<<TWEN); 
	
    while (!(TWCR & (1<<TWINT)));   		
		  								
    if ((TWSR & 0xF8) == TWI_START)		
 	   return(0);
	else
	   return(1);
}
//===============================================
unsigned char TWI_RepeatStart(void)
{
 
    TWCR = (1<<TWINT)|(1<<TWSTA)|(1<<TWEN); 
    while (!(TWCR & (1<<TWINT)));   		
		  								
    if ((TWSR & 0xF8) == TWI_REPEAT_START)		
 	   return(0);
	else
	   return(1);
}
//===============================================
unsigned char TWI_SendAddr(unsigned char addrs)
{
   unsigned char STATUS;
   
   if((addrs & 0x01) == 0) 
     STATUS = TWI_MASTER_TX_SLAVE_ACK;
   else
     STATUS = TWI_MASTER_RX_SLAVE_ACK; 
   
   TWDR = addrs; 
   TWCR = (1<<TWINT)|(1<<TWEN);	 
   		  			 			
   while (!(TWCR & (1<<TWINT)));
   		 		   				
								
   if ((TWSR & 0xF8) == STATUS)	
   	  return(0);
   else 
      return(1);
}
//===============================================
unsigned char TWI_SendData(unsigned char data)
{
   TWDR = data; 
   TWCR = (1<<TWINT) |(1<<TWEN);	  
   		  			 				   
   while (!(TWCR & (1<<TWINT)));	   
   		 		   					   
									   
   if ((TWSR & 0xF8) != TWI_MASTER_TX_DATA_ACK) 
   	  return(1);
   else
      return(0);
}
//===============================================
unsigned char TWI_ReceiveDataWithAck(void)
{
  unsigned char data;
  
  TWCR = (1<<TWEA)|(1<<TWINT)|(1<<TWEN);
  
  while (!(TWCR & (1<<TWINT)));	   
   		 		   				
  if ((TWSR & 0xF8) != TWI_MASTER_RX_DATA_ACK) 
   	  return(TWI_ERROR_CODE);
  
  data = TWDR;
  return(data);
}
//===============================================
unsigned char TWI_ReceiveDataWithNoAck(void)
{
  unsigned char data;
  
  TWCR = (1<<TWINT)|(1<<TWEN);
  
  while (!(TWCR & (1<<TWINT)));	  
   		 		   				
  if ((TWSR & 0xF8) != TWI_MASTER_RX_DATA_NACK)    
   	  return(TWI_ERROR_CODE);
  
  data = TWDR;
  return(data);
}
//===============================================
void TWI_Stop(void)
{
  TWCR =  (1<<TWINT)|(1<<TWEN)|(1<<TWSTO);	  
}  
//===============================================
#endif


