/*
file name : GroupMessenger.C
author : fhn
date : 04-FEB-2019
MCU : ATmega32

parts : LCD=16x2   RTC=DS1307   WIRELESS=HC-12
*/

//=============================================================
// user defined names and messages

char USER_NAMES[8][11] = { "UserName01", "UserName02", "UserName03", "UserName04", 
						  "UserName05", "UserName06", "UserName07", "UserName08" };
char *USER_MESSAGES[] = { "Send e-mail", "Attend Meeting", "Contact Me", "Call Back",
					   "Job Status ?", "OK / Done", "I am Leaving", "On Leave Tomorow",
					   "Bring File", "Contact Client", "Contact Head","Get Application",
					   "Ready.","In Progress...","Waiting...","Logged In"};

//=============================================================
#define USART_BUF_SIZ 40
#define RECD_BUF_SIZ 10
char USART_BUF[USART_BUF_SIZ+1];
char RDATA_BUF[RECD_BUF_SIZ][USART_BUF_SIZ];
char TEMP_STR[USART_BUF_SIZ+10];

#define  F_CPU 1000000UL 
#define  FCPU_BY_100  F_CPU/100

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>

#include <string.h>
#include "FHN_LCD4_LIB.C"
#include "FHN_EEPROM_LIB.C"
#include "FHN_TWI_LIB.C"
#include "FHN_RTC_DS1307_LIB.C"



#define SEL_BTN_PORT		PINA

#define EXE_BTN_PORT	PINB
#define BTN_EXE_BITS	0b00011111
#define BTN_SEND		0b00000001
#define BTN_DELETE		0b00000010
#define BTN_UP			0b00000100
#define BTN_DOWN		0b00001000
#define BTN_NEXT		0b00000001
#define BTN_PREV		0b00000010
#define BTN_CONF		0b00010000

#define BLINKER_LED 	0b10000000 
#define NEW_MSG_LED 	0b01000000 

#define BUZZER_BIT 		0b01000000 
#define HC12_SET		0b00000100 
#define ID_BITS			0b00111000 

#define ON  	1
#define OFF 	0
#define TOGGLE	2
#define ERROR  	1
#define OK		0

#define PRCS_CLOCK	0
#define PRCS1_SEL	1
#define PRCS2_SEL	2
#define PRCS3_SEL	3
#define PRCS_EXECUTE	9

uint8_t CUR_PRCS = 0;


unsigned char USART_RXD=0;
uint8_t USART_RDP = 0;
int8_t  BUF_RECV_POS = 0;
int8_t  BUF_VIEW_POS = -1;
uint8_t USART_MY_ADDR = 100;
uint8_t USART_TO_ADDR = 100;
uint8_t USART_READY=0;

uint8_t MY_ID_NO=32;
uint8_t SEL_ID_NO=100;
uint8_t SEL_MSG=100;
uint8_t isNewMsg=0;

uint8_t errUSART, errHC12, errRTC;
uint8_t HC12_AT=0;
//============================================================
void initPorts ( )
{
	DDRA=0;			  // 8 selection switches
	DDRB=0b11000000;  // 2 LEDS upto 5 execute switches
	DDRC=0b11111101;	  // 6 pins for LCD + DS1307
	DDRD=0b01000110;  //clock-blink + buzzer + 3 dip swithes + HC12_SET + usart(for HC12)
	
	cli(); 
}
//============================================================
int USART_init ( )
{

	UBRRL = 12;
	UBRRH = 0;

	UCSRA |= (1<<U2X);  
	UCSRB = ( (1<<RXEN) | (1<<TXEN) | (1 << RXCIE));
	UCSRC =  ((1<<URSEL) | (3<<UCSZ0) );  

	return ( 0 );
}  
//==================================================================
int USART_start ( )
{
	
	LCD_clr();
	LCD_wrStr("Starting USART:" );
	int res = USART_init ( );
	LCD_RC(2,1);
	if (res==0)
		LCD_wrStr( "9600-8-N-1");
	else
	{
		LCD_wrStr("ERROR");
		_delay_ms (2000);
	}
	_delay_ms (100);
	return res;
}
//=============================================================================
char HC12_sendChar ( char ch )
{
	while ( ! (UCSRA & (1<<UDRE) ) ) 
	{
		_delay_us ( 1 ); 
	}
	_delay_us ( 100 );
	UDR = ch;
	_delay_us ( 10 );
	return ( 0 );
}
//=============================================================================
void HC12_sendStr ( char *str , uint8_t isNew)
{
	if (isNew)
	{
		USART_RDP = 0;
		USART_READY=0;
	}

	PORTD |= HC12_SET;
	for ( int i=0; str[i]!=0; i++ )
		HC12_sendChar ( str[i] );
}
//=============================================================================
void HC12_sendCom ( char *command )
{
	HC12_AT=1;
	PORTD &= (~HC12_SET);
	_delay_ms ( 100 );
	USART_READY=0;
	USART_RDP=0;
	for ( int i=0; command[i]!=0; i++ )
		HC12_sendChar ( command[i] );
	_delay_ms ( 100 );
	PORTD |= HC12_SET;
	HC12_AT=0;
}
//=============================================================================
int HC12_start ( )
{
	LCD_clr();
	LCD_wrStr("Testing HC-12:");
	LCD_RC(2,1);
	
	HC12_sendCom ( "AT" );
	
	
	if ( USART_BUF[0] == 'O' && USART_BUF[1] == 'K' )
		LCD_wrStr( "OK"); 
	else
	{
		LCD_wrStr("ERROR");
		_delay_ms (2000);
		return ERROR;
	}
	USART_RDP = 0;
	USART_READY=0;

	_delay_ms (100);
	return OK;
}
//=============================================================================
void setBlinker ( uint8_t status )
{
	if (status==ON)
		PORTB |= (BLINKER_LED);
	else if (status==TOGGLE)
		PORTB ^= (BLINKER_LED);
	else // off
		PORTB &= (~BLINKER_LED);
}
//============================================================
void setBuzzer ( uint8_t status )
{
	if (status)
		PORTD |= (BUZZER_BIT);
	else // off
		PORTD &= (~BUZZER_BIT);
}
//============================================================
void Beep ( )
{
	setBuzzer ( ON );
	_delay_ms ( 250 );
	setBuzzer ( OFF );
}
//============================================================
void setNewMsgLed ( uint8_t status )
{
	if (status)
		{
		PORTB |= (NEW_MSG_LED);
		isNewMsg = 1;
		}
	else // off
		{
		PORTB &= (~NEW_MSG_LED);
		isNewMsg=0;
		}
}
//============================================================
void showDateTime()
{

	CUR_PRCS = PRCS_CLOCK;
	LCD_clr();
	if (errRTC)
	{
		LCD_RC ( 1,4);
		LCD_wrStr ( "RTC IC : ERROR" );
		return;
	}

	RTC_getDate();
	LCD_RC ( 1,4);
	LCD_wrStr ( RTC_DAT_STR );

	RTC_getTime();
	LCD_RC ( 2,5 );
	LCD_wrStr ( RTC_TIM_STR );
}
//============================================================
void selProcss ( int btncod )
{
	int p=0;
	int btnval = btncod;
	if ( CUR_PRCS == PRCS1_SEL )
		CUR_PRCS = PRCS2_SEL;
	else if ( CUR_PRCS == PRCS2_SEL )
		CUR_PRCS = PRCS3_SEL;
	else //if ( CUR_PRCS == PRCS3_SEL ) or first time entry
		CUR_PRCS = PRCS1_SEL;

	for (p=0; (btnval & 0b00000001)==0; btnval=btnval>>1)
	{ p++; };

	if ( CUR_PRCS == PRCS1_SEL )
		SEL_ID_NO = p;

	if ( CUR_PRCS == PRCS2_SEL )
		SEL_MSG = p;

	if ( CUR_PRCS == PRCS3_SEL )
		SEL_MSG = p+8;

	LCD_clr();
	LCD_RC(1,1);
	LCD_wrStr( USER_NAMES[SEL_ID_NO] );
	LCD_RC(2,1);
	if ( SEL_MSG < 16 )
		LCD_wrStr( USER_MESSAGES[SEL_MSG] );

	_delay_ms ( 500 );
}
//============================================================
uint8_t readRecvData ( )
{

	if ( USART_READY==1 )
	{
		if ( USART_BUF[0] != 27 || USART_BUF[1] != (MY_ID_NO+1) )
		{
			USART_RDP = 0;
			USART_READY=0;
			return 0;
		}

		
		if ( BUF_RECV_POS>=(RECD_BUF_SIZ) )
		{
			for ( uint8_t j=0; j<(RECD_BUF_SIZ-2); j++ )
				strcpy ( RDATA_BUF[j] , RDATA_BUF[j+1] );
			BUF_RECV_POS = RECD_BUF_SIZ-1;
		}
		
		uint8_t n=0;
		for ( n=0; USART_BUF[n]!=26 && USART_BUF[n]!=0; n++ ) 
			RDATA_BUF[BUF_RECV_POS][n]=USART_BUF[n];
		RDATA_BUF[BUF_RECV_POS][n]=0;

	
		
		setNewMsgLed ( ON );
		setBuzzer ( ON );
		strncpy ( TEMP_STR , &USART_BUF[3], 10 );
		TEMP_STR[10] = 0;
		EEPROM_WriteString ( USART_BUF[2]*20, TEMP_STR, 11 );
		USART_RDP = 0;
		USART_READY=0;
		BUF_VIEW_POS=BUF_RECV_POS;
		BUF_RECV_POS++;
		_delay_ms ( 800 );
		setBuzzer ( OFF );
		return 1;
	}
	else
		return 0;
}
//============================================================
void sendMessage()
{

	if (errUSART>0 || errHC12>0)
	{
		LCD_wrStr ( "USART/HC12:ERROR" );
		_delay_ms ( 1000 );
		return;
	}
	if ( SEL_ID_NO == MY_ID_NO )
	{
		LCD_wrStr ( "MESSAGE CANNOT");
		LCD_RC ( 2,2 );
		LCD_wrStr (  "SEND TO SELF" );
		_delay_ms ( 2000 );
		return;
	}

	if ( SEL_ID_NO > 16 )
	{
		LCD_wrStr ( "RECEIVER NAME");
		LCD_RC ( 2,2 );
		LCD_wrStr (  "NOT SELECTED." );
		_delay_ms ( 2000 );
		return;
	}

	if ( SEL_MSG > 16 )
	{
		LCD_wrStr ( "MESSAGE ");
		LCD_RC ( 2,2 );
		LCD_wrStr (  "NOT SELECTED." );
		_delay_ms ( 2000 );
		return;
	}
	LCD_clr();
	LCD_wrStr ( USER_NAMES[SEL_ID_NO] );
	LCD_RC ( 2,1 );
	LCD_wrStr ( USER_MESSAGES[SEL_MSG]);
	RTC_getTime();
	LCD_RC ( 1,12 );
	LCD_wrStr ( RTC_TIM_STR );
	_delay_ms ( 500 );
	
	for (uint8_t n=0; USART_RDP > 0 ; n++ )
	{
		_delay_ms ( 10 );
		readRecvData ( );
		if (n>500)
			break;
	}

	USART_RDP = 0;
	USART_READY=0;

	PORTD |= HC12_SET;
	_delay_ms(100);
	HC12_sendChar ( 27 );
	_delay_us ( 100 );
	HC12_sendChar ( SEL_ID_NO+1 );
	_delay_us ( 10 );
	HC12_sendChar ( MY_ID_NO+1 );
	HC12_sendStr ( USER_NAMES[MY_ID_NO] , 0);
	HC12_sendChar ( 32 );
	HC12_sendStr ( RTC_TIM_STR , 0);
	HC12_sendChar ( 22 );
	HC12_sendStr ( USER_MESSAGES[SEL_MSG], 0 );
	HC12_sendChar ( 26 );
	HC12_sendChar ( 0 ); 
	HC12_sendChar ( 0 ); 
	
	LCD_clr();
	LCD_wrStr ( "MESSAGE SENT");
	LCD_RC ( 2,3 );
	LCD_wrStr (  "SUCCESSFULLY" );
	SEL_MSG = 100;
	SEL_ID_NO = 100;
	_delay_ms ( 2000 );
	return;
	
}
//============================================================
uint8_t showRecvMsg ( uint8_t msgno )
{
	LCD_clr();

	if ( BUF_VIEW_POS==-1 )
	{
		LCD_wrStr ( "NO MESSGES FOUND");
		LCD_RC ( 2,1 );
		LCD_wrStr ( "OR BUFFER EMPTY." );
		_delay_ms(1000);
		return 1;
	}
	
	LCD_wrStr ( &RDATA_BUF[BUF_VIEW_POS][3]);
	LCD_RC ( 2,1 );
	LCD_wrStr ( &RDATA_BUF[BUF_VIEW_POS][23] );
	return 0;


}
//============================================================
void mangMsg( int process )
{

	if (showRecvMsg ( BUF_VIEW_POS )) // buffer is empty error
		return;

	
	if ( process==0 ) 
	{
		LCD_RC ( 1, 11 );
		LCD_wrStr ( "DELETE" );
		_delay_ms(1500);

		for ( uint8_t j=BUF_VIEW_POS; j<(RECD_BUF_SIZ-2); j++ )
				strcpy ( RDATA_BUF[j] , RDATA_BUF[j+1] );
		if ( BUF_RECV_POS>0 )
			BUF_RECV_POS--;
		if ( BUF_VIEW_POS> 0 )
			BUF_VIEW_POS--;
		if ( BUF_RECV_POS<=0 ) 
		{
			BUF_RECV_POS=0;
			BUF_VIEW_POS=-1;
		}

	}
	else if ( process==-1 ) // up
	{
		
		if ( BUF_VIEW_POS>0 )
			BUF_VIEW_POS--;
		else
			Beep ( );
	}
	else if ( process==1 ) 
	{
		if ( BUF_VIEW_POS<(BUF_RECV_POS-1) )
			BUF_VIEW_POS++;
		else
			Beep ( );
	}
	

	showRecvMsg ( BUF_VIEW_POS );
	return;
}
//============================================================
void editUserName ( )
{

	int  ch=0;
	uint8_t eloop=0, pos=0, isEdited=0;

	LCD_wrCom ( 0x0F );
	LCD_clr();
	
	LCD_wrStr ( "User Name : " );

	strcpy ( TEMP_STR, USER_NAMES[MY_ID_NO] );
	
	LCD_RC (2,1 );
	LCD_wrStr ( TEMP_STR  );
	
	LCD_RC (2,1 );
	_delay_ms ( 3000 );
	while (eloop<20)
	{
		LCD_RC ( 2, pos+1 );
		if (eloop==0)
		_delay_ms (1000);
		else
		_delay_ms (300);
		
		
		if ( EXE_BTN_PORT & BTN_UP) 
		{
			eloop=0;
			isEdited=1;
			if (pos<10)
			{
				ch = TEMP_STR[pos];
				ch++; 
				if (ch>125) ch=32;
				TEMP_STR[pos] = ch;
				LCD_RC (2,1 );
				LCD_wrStr ( TEMP_STR );
			}
		}
		else if ( EXE_BTN_PORT & BTN_DOWN ) 
		{
			eloop=0;
			isEdited=1;
			if (pos<10)
			{
				ch = TEMP_STR[pos];
				ch--; 
				if (ch<32) ch=125;
				TEMP_STR[pos] = ch;
				LCD_RC (2,1 );
				LCD_wrStr ( TEMP_STR );
			}
		}
		else if ( EXE_BTN_PORT & BTN_NEXT ) 
		{
			eloop=0;
			pos++;
			if (pos>9)
				pos=0;
		}
		else if ( EXE_BTN_PORT & BTN_PREV ) 
		{
			eloop=0;
			pos--;
			if (pos<0)
				pos=9;
		}
		else
		{
			eloop++;
		}
	}

	LCD_wrCom ( 0x0C ); 

	if ( isEdited==1 )
	{
		strcpy ( USER_NAMES[MY_ID_NO] , TEMP_STR );
		EEPROM_WriteString ( (MY_ID_NO+1)*20, TEMP_STR, 11 );
		LCD_RC ( 2, 12 );
		LCD_wrStr ("SAVED");
		_delay_ms ( 2000 );
	}



}
//============================================================
void editDateTime ( )
{
	int  ch=0;
	uint8_t eloop=0, pos=0, isEdited=0;

	LCD_wrCom ( 0x0F ); 
	LCD_clr();

	RTC_getDate();

	LCD_wrStr ( RTC_DAT_STR );
	if (RTC_DAT_STR[6]=='Y' )
		RTC_DAT_STR[6]='2';
	if (RTC_DAT_STR[7]=='Y' )
		RTC_DAT_STR[7]='0';

	RTC_getTime();
	LCD_RC (2,1 );
	LCD_wrStr ( RTC_TIM_STR );
	
	LCD_RC (1,1 );
	_delay_ms ( 3000 );
	while (eloop<20)
	{
		LCD_RC ( pos/10+1, pos%10+1 );
		if (eloop==0)
		_delay_ms (1000);
		else
		_delay_ms (300);
		
		
		if ( (EXE_BTN_PORT & BTN_UP) )
		{
			eloop=0;
			isEdited=1;
			if (pos<10)
			{
				ch = RTC_DAT_STR[pos%10];
				ch++; if (ch>'9') ch = '0';
				if ( ch>'3' && pos==0 )
					ch='0';
				if ( ch>'1' && (pos==3 || pos==7) )
					ch='0';
				if ( ch>'1' && RTC_DAT_STR[0]=='3' && pos==1 )
					ch='0';
				if ( ch>'2' && RTC_DAT_STR[3]=='1' && pos==4 )
					ch='0';
				RTC_DAT_STR[pos%10] = ch;
				LCD_RC (1,1 );
				LCD_wrStr ( RTC_DAT_STR );
			}
			else
			{
				ch = RTC_TIM_STR[pos%10];
				ch++;	if (ch>'9')	ch = '0';
				if ( ch>'5' && (pos==13 || pos==16) )
					ch='0';
				if ( ch>'2' && pos==10 )
					ch='0';
				if ( ch>'3' && RTC_TIM_STR[0]=='2' && pos==11 )
					ch='0';
				RTC_TIM_STR[pos%10] = ch;
				LCD_RC (2,1 );
				LCD_wrStr ( RTC_TIM_STR );
			}
		}
		else if ( (EXE_BTN_PORT & BTN_NEXT) ) 
		{
			//isEdited=1;
			eloop=0;
			pos++;
			if (pos>=18)
				pos=0;
			if (pos==2 || pos==5 || pos==12 || pos==15 )
				pos++;
			if (pos==6) 
				pos++;
			if (pos==7) 
				pos++;
		}
		else
		{
			eloop++;
		}
	}

	if ( isEdited==1)
	{
	if ( RTC_DAT_STR[0]=='3' && RTC_DAT_STR[1]>'1')
		{ RTC_DAT_STR[1]='1'; }
	if ( RTC_DAT_STR[3]=='1' && RTC_DAT_STR[4]>'2')
		{ RTC_DAT_STR[4]='2'; }
	if ( RTC_DAT_STR[0]=='3' && RTC_DAT_STR[3]=='0' && RTC_DAT_STR[4]=='2')
		{ RTC_DAT_STR[0]='2'; RTC_DAT_STR[1]='8'; }
	if ( RTC_DAT_STR[0]=='3' && RTC_DAT_STR[1]>'0' && RTC_DAT_STR[3]=='0' &&(RTC_DAT_STR[4]=='4' || RTC_DAT_STR[4]=='6' || RTC_DAT_STR[4]=='9' ))
		{ RTC_DAT_STR[1]='0'; }
	if ( RTC_DAT_STR[0]=='3' && RTC_DAT_STR[1]>'0' && RTC_DAT_STR[3]=='1' && RTC_DAT_STR[4]=='1')
		{ RTC_DAT_STR[1]='0'; }
	RTC_WriteDate ( RTC_DAT_STR );
	RTC_WriteTime ( RTC_TIM_STR );
	LCD_RC (2,11 );
	LCD_wrStr ("SAVED");
	_delay_ms ( 2000 );
	}
	LCD_wrCom ( 0x0C ); 

}
//============================================================
void exeProcss ( int btncod )
{
	CUR_PRCS = PRCS_EXECUTE;
	LCD_clr();

	if ( btncod&BTN_SEND )
		sendMessage();
	else if ( btncod&BTN_DELETE )
		mangMsg(0);
	else if ( btncod&BTN_UP )
		mangMsg(-1);
	else if ( btncod&BTN_DOWN )
		mangMsg(1);
	else if ( btncod&BTN_CONF )
	{
		editUserName();
		editDateTime();
	}
	else
		LCD_wrStr("INVALID BUTTON. ");

	_delay_ms ( 1000 );
}

//============================================================
uint8_t	EEPROM_start ( )
{
	uint8_t r=0;
	EEPROM_ReadString ( 0, TEMP_STR, 4 );
	if ( strcmp (TEMP_STR,"fhn" ) != 0 ) 
	{
		EEPROM_WriteString ( 0, "fhn\0", 5 );
		for (uint8_t i = 1; i<=8; i++ )
		{
			_delay_ms (10);
			TEMP_STR[9] = i+'0';
			EEPROM_WriteString ( i*20, USER_NAMES[i-1], 11 );
		}
		r=1;
	}
	_delay_ms (10);
	for (uint8_t i = 1; i<=8; i++ )
		{
			EEPROM_ReadString ( i*20, USER_NAMES[i-1], 11 );
			_delay_ms (5);
		}

	return r;
}
//============================================================
int main ( ) 
{
	int loopno=0;
	int btncod=0;
	

	MCUCSR |= 0x80;
	MCUCSR |= 0x80;
	
	_delay_ms ( 100 );

	initPorts ();
	_delay_ms ( 100 );

	for ( uint8_t x=0; x<RECD_BUF_SIZ; x++ )
		RDATA_BUF[x][0]=0;

	setBlinker (ON);
	_delay_ms ( 500 );
	setBlinker (OFF);
	_delay_ms ( 500 );
	
	LCD_start ( );
	_delay_ms ( 100 );

	LCD_clr();
	LCD_wrStr("GROUP MESSENGER");
	LCD_RC (2, 10);
	LCD_wrStr("-fhn/..");

	
	setBlinker (ON);
	_delay_ms ( 1000 );
	setBlinker (OFF);
	_delay_ms ( 1000 );	

	LCD_clr();
	LCD_wrStr("CHECK INTERNAL" );
	LCD_RC (2, 1);
	LCD_wrStr("MEMORY : ");

	uint8_t e = EEPROM_start ( );
	
	if (e)
		LCD_wrStr("NEW");
	else
		LCD_wrStr("OK");

	setBlinker (ON);
	_delay_ms ( 1000 );
	setBlinker (OFF);
	_delay_ms ( 1000 );	


	MY_ID_NO = (PIND & ID_BITS)>>3;
	LCD_clr();
	LCD_wrStr("MY ID No:");
	LCD_wrChr ( MY_ID_NO+'1');
	LCD_RC (2, 1);
	LCD_wrStr( USER_NAMES[MY_ID_NO] );

	setBlinker (ON);
	setNewMsgLed ( ON );
	_delay_ms ( 2000 );
	setBlinker (OFF);
	setNewMsgLed ( OFF );

	Beep();

	sei();

	errUSART = USART_start ( );
	
	setBlinker (ON);
	_delay_ms ( 500 );
	setBlinker (OFF);
	_delay_ms ( 500 );

	if (errUSART!=OK)
		goto RTC;


	errHC12 = HC12_start ( );

	setBlinker (ON);
	_delay_ms ( 500 );
	setBlinker (OFF);
	_delay_ms ( 500 );

			
	RTC:
	
	_delay_ms ( 100 );
	LCD_clr();
	LCD_wrStr("Check RTC:DS1307");
	LCD_RC (2, 1);
	errRTC=RTC_getTime ( );
	strcpy (TEMP_STR, RTC_TIM_STR );
	if ( errRTC ) 
		LCD_wrStr( "ERROR" ); 
	else if (strcmp ( RTC_TIM_STR, "HH:MM:SS")==0)
		{
		LCD_wrStr( "NOT CONNECTED" );
		errRTC = 2;
		}
	else 
		{
		int v = (PIND & 0x80)>>7;
		_delay_ms ( 400 );
		v |= (PIND & 0x80)>>6;
		_delay_ms ( 400 );
		v |= (PIND & 0x80)>>5;
		_delay_ms ( 400 );
		v |= (PIND & 0x80)>>4;
		RTC_getTime ( );
		if ( strcmp (TEMP_STR, RTC_TIM_STR)==0 || v==0 || v==0x0F)
			{
			LCD_wrStr( "NOT INITIATED" );
			_delay_ms ( 1000 );
			RTC_WriteTime( "10:15:20");
			_delay_ms ( 1000 );
			RTC_WriteDate("01-01-2020");
			LCD_RC (2, 1);
			LCD_wrStr( "SET TO DEFUALTS" );
			_delay_ms ( 1000 );
			}
		LCD_wrStr( RTC_TIM_STR );
		}


	_delay_ms ( 1500 );

	while ( 1 )
	{
		setBlinker ( TOGGLE );

		if ( loopno<0 )
		{
			
			if (readRecvData ( ) )
			{
				showRecvMsg ( BUF_VIEW_POS );
				loopno = 100;
			}
			else if (!isNewMsg)
			{
				showDateTime();
				loopno = 18;
			}
		}
		
		if ( (btncod=SEL_BTN_PORT) != 0 )
		{
			setNewMsgLed ( OFF );
			selProcss ( btncod );
			loopno = 80;
		}

		if ( (btncod=(EXE_BTN_PORT&BTN_EXE_BITS)) != 0 ) //BTN_CONTROL_BITS )
		{
			setNewMsgLed ( OFF );
			exeProcss ( btncod );
			loopno = 40;
		}

		_delay_ms ( 50 );
		loopno--;
	} // end of while
	
	return 0;
}
//============================================================
ISR ( USART_RXC_vect )
{
	USART_RXD = UDR; 
	
	if ( USART_RDP==0 && USART_RXD !=27 && HC12_AT==0) // MATCH THE BEGIN OF RECEPTION
	{
		USART_RDP=0;
		USART_READY=0;
		return;
	}
	
	if ( USART_RXD == 26 ) // END OF RECEPTION
		USART_READY=1;
	
	if ( USART_RDP < USART_BUF_SIZ && USART_RXD>0)
			USART_BUF[USART_RDP++] = USART_RXD;

}
//============================================================






